# flake8: noqa: F401
# import recipes here so they get are defined and available
from .rcp_catchall import CatchAllRecipe
from .rcp_oah import OAHRecipe
from .rcp_qlsi import QLSIRecipe
from .rcp_rtdc import RTDCRecipe
